% User Guide for `42`
% Denis BAURAIN [<denis.baurain@ulg.ac.be>]
% Version 0.153130 / Nov 21, 2015


# Aim and features

The aim of `42` is to add (and to optionally align) sequences to a preexisting multiple sequence alignment (MSA) while controlling for orthology relationships and potentially contaminating sequences.

Sequences to add are either nucleotide transcripts resulting from transcriptome assembly or already translated protein sequences. In theory, one can also use genomic nucleotide sequences (because `42` can splice introns), but this possibility has not been extensively tested so far.

The working hypothesis of `42` (not yet demonstrated) is that its orthology-controlling heuristics can enrich not only MSAs of single-copy genes but also more complicated MSAs including terminally duplicated genes (in-paralogues) and/or corresponding to multigenic families featuring different out-paralogues of different ages. Preliminary tests on a broadly sampled eukaryotic data set suggest that the orthology relationships enforced by `42` are in good agreement with those inferred with *OrthoFinder* software [Emms and Kelly (2015) *Genome Biol* 16:157].

Regarding contaminations, `42` implements a system of taxonomic filters (based on *NCBI Taxonomy*) allowing it to flag (or to reject) any new sequence for which the taxonomic affiliation is doubtful. However, the power of this mechanism is currently dependent on the taxonomic breadth of each MSA.

`42` is exclusively setup through a structured text file (e.g., *YAML* format). Storage of this file allows a user to document all the configuration details for a given run. An example of a complete `config` file is available in [Annotated YAML `config` file](#annotated-yaml-config-file).

`42`'s verbosity is configured directly on the command line. `42` can be very introspective if asked to be so. At the highest verbosity level, the numerous `BLAST` reports are not deleted after the run and are thus available for manual inspection (e.g., for debugging purposes).


# Design principles

In a single run, `42` can process an arbitrary large number of MSAs (specified using shell jokers on the command line). Moreover, one can search for orthologous sequences in as many organisms as wanted.

The configuration (`config`) file has two main parts : one with the options that apply globally to the run and one that lists the organisms (`orgs`) to search and their specific options, including the path (`bank_dir`) to the corresponding sequence databases (`banks` in `42`'s parlance). The `config` file includes a mechanism of default values (`defaults`) that apply to all organisms except when otherwise specified in individual `org` subsections (e.g., `bank_type`, `code`).

When `42` enriches a MSA, it processes each organism in turn following the order of `org` subsections in the `config` file. Several *out-of-order* optimisations ensure that similar computations (e.g., `BLAST` searches) are not repeated uselessly.


# Functional overview

A graphical overview of `42`'s pipeline is available at next page (Figure 1).

![Overview of `42`'s pipeline (see text for details).](forty-two-scheme.pdf)


## Orthology-controlling heuristics

Each run of `42` must specify a set of reference organisms (`ref_orgs`), for which the complete proteomes have to be available (`ref_bank_dir`, `ref_org_mapper`), and a set of query organisms (`query_orgs`), which should be represented in most MSAs to be enriched. These two sets of organisms do not need to be identical but certainly can. They will apply to all organisms (`orgs`) to be added to yield the new MSAs (`out_suffix`).


### Collection of queries

For each `org`, `42` extracts *all* sequences belonging to the `query_orgs` in order to assemble a list of `query_seqs`. If a sequence is flagged as a contamination (`c#`) or as a diverging sequence (`d#`), it is automatically excluded. <!-- Il n'y a pas d'explications de la phylosophie de naming de BMC donc le principe du flag n'est pas forcément clair ici (début ou fin de nom; _, - ou blanc) --> Eventually, if a MSA does not contain any sequence fulfilling the selection criteria, `42` warns the user and falls back to selecting the longest sequence instead, which leads to a singleton `query_seqs`.


### Preflight check of orthology relationships

To ensure that it can accurately enrich MSAs in orthologous sequences, `42` verifies that `query_seqs` and `ref_orgs` themselves satisfy its orthology criteria. This two-step process is carried out separately for each MSA.

First, an average `BLASTP` bit score is computed for each `ref_org` based on the individual best hits of each `query_seq` against the corresponding complete proteomes. `query_seqs` without any hit in a given `ref_org` are taken into account by contributing a value of zero to the average bit score for the `ref_org`. How exactly first hits are considered best hits is explained in [Identification of best hits for queries](#identification-of-best-hits-for-queries).

`ref_orgs` without any hit to `query_seqs` are automatically discarded, whereas the remaining ones are ranked in descending order on the average bit score. Low-scoring `ref_orgs` can be optionally discarded by specifying a value < 1.0 for the `ref_org_mul` parameter of the `config` file. For example, assuming the user lists 10 different `ref_orgs` and set `ref_mul_org` to 0.7, at most 7 `ref_orgs` will be retained for assessing orthology relationships. This could be the result of the automatic removal of two `ref_orgs` without any hit and of an additional low-scoring one to honor the `ref_mul_org` setting.

Second, the best hits for each `ref_org` are `BLAST`ed (`BLASTP`) against the complete proteomes of other `ref_orgs` to check that they indeed recover the same best hits as the `query_seqs`. If any `ref_org` fails with any of the other `ref_orgs`, a message is issued to warn the user, but `42` proceeds normally. More details about the logic behind this are available in [Identification of orthologues among homologues](#identification-of-orthologues-among-homologues). Otherwise, the preflight check is considered successful.


### Search for homologues using queries

Each one of the `query_seqs` is `BLAST`ed in turn against each one of the `banks` for the current `org`. The exact `BLAST` flavour is either `TBLASTN` or `BLASTP`, depending on the sequence type of `org`'s `banks`. Moreover, default options of this first `BLAST` can be overridden by specifying key/value pairs in the subsection `homologues` under the section `blast_args` of the `config` file (e.g., low-complexity filters, E-value threshold, maximum number of hits). <!-- Ici, tu expliques qu'il est possible de modifier le comportement de BLAST pour ce premier blast. D'abord, je considère que ce n'est pas le premier blast vu que les BRH se font en amont. Ensuite, vu que cela s'applique sur chaque blast, je pense que tu peux faire un encart pour préciser que BLAST est complétement configurable -->

The whole set of hits corresponding to all `query_seqs` is consolidated into a single list of **homologous** sequences. These sequences can be optionally trimmed to the segment really covered by the matching `query_seqs`. This behaviour is useful to avoid non-core regions to perturb orthology assessment. It is controlled by the `seq_trimming` parameter of the `config` file.


### Identification of best hits for queries

Each `query_seq` is furthermore `BLAST`ed (`BLASTP`) against the complete proteome of each `ref_org`. <!-- Cette partie me semble très redondante avec le preflight et je me demande si les 3 morceaux après le preflight ne devrais pas etre positionné un rang en dessous. Ou alors, je n'ai pas compris. --> Again, `BLAST` options can be overridden if needed (subsection `references` under section `blast_args`). For each `query_seq`, the best hit in the `ref_org` is recorded. However, when bit scores of subsequent hits are nearly equal to the bit score of the best hit, the corresponding sequences are interpreted as closely related in-paralogues and also added to the list of **best hits**. This behaviour can be tweaked using the `bitscore_mul` parameter of the `config` file.

As a consequence, several best hits can be recorded for a single `query_seq`/`ref_org` pair, either because several sequences are available for the `query_org` (in-paralogues or out-paralogues in the case of a multigenic family) or because several sequences match a single `query_seq` in the `org`'s `banks` (which should be co-orthologues then), or for both reasons. In contrast, if a `ref_org` has no homologue for the current MSA, `42` warns the user and drops it from the list of `ref_orgs` considered by the orthology-controlling engine. <!-- Il faudrait préciser que le retrait des ref_orgs dont tu parles ici n'est pas influencé par ref_org_mul. En gros, l'explication du choix des ref_orgs me semble un peu dispersé -->


### Identification of orthologues among homologues

To sort out orthologous sequences from paralogous sequences, each homologue in the current `org` is `BLAST`ed (`BLASTX` or `BLASTP`) against the complete proteome of each `ref_org` (`BLAST` options in subsection `orthologues` under section `blast_args`). And now, here's the heart of `42`'s heuristics... To be considered as an orthologue, a homologue must satisfy the following criterion for every one of the (active) `ref_orgs` without exception: its best hit in the corresponding complete proteome must be found in the original list of best hits assembled using the `query_seqs`.

It is important to note that `42` does not care at all about which particular `query_seq` (or `query_seqs`) recovered the homologue in the `org` nor about those that recovered the best hits in the complete proteomes of the `ref_orgs`. The only thing that matters is that *the loop is closed*. The set of homologues for which this condition holds then become the **orthologues**. If the parameter `brh_mode` of the `config` file is set to `disabled`, all homologues are automatically considered as orthologues.


## Orthologue post-processing

Once orthologues are identified, each one is `BLAST`ed (`BLASTX` or `BLASTP`) against the MSA itself to recover its closest relatives (`BLAST` options in subsection `templates` under section `blast_args`).


### Family affiliation and orthologue naming

If the most closely related sequence in the MSA belongs to a given family (e.g., `mt-`), the orthologue is affiliated to the same family, as did the original `forty`. This allows enriching MSAs corresponding to multigenic families. Note that only the most closely related sequence can be used to infer the orthologue's family.

The orthologue identifier is built using the `org` name and the accession of the corresponding sequence in the `org`'s `banks`, which helps tracking down all the sequences added to a MSA by `42` (e.g., for debugging purposes). This is thus different from the original `forty`, in which most sequences were *contigs* having lost all connection with the nucleotide sequences in the `org`'s `banks`.


### Contamination detection and handling

`42` then seeks to determine whether the orthologue is a genuine orthologue or a xenologue contaminating the `org`'s `banks`. To this end, it infers the orthologue's taxonomy by analysing the identifiers of the five closest sequences in the MSA. More precisely, it considers each of them in turn and stops as soon as one of them can be reliably affiliated to a *NCBI Taxonomy* entry.

If the taxon corresponding to the entry satisfies the taxonomic filter (`tax_filter` parameters in the `config` file), the orthologue is added to the MSA. Otherwise, it is either removed or added but tagged as a contamination (`c#`), depending on the value of the `tf_action` parameter. When an orthologue is tagged as a contamination, the binomial of the organism at the origin of the taxonomic inference is appended to its identifier (i.e., `...Genus_species`).

Taxonomic filters are optional and, if used, they require a local copy of the *NCBI Taxonomy* database (`tax_dir` parameter in the `config` file). It can be installed using `setup-taxdir.pl`. Taxonomic identification based on organism names is always possible and can be made less strict for binomial names that are not yet available as a *NCBI Taxonomy* entry. This is controlled by the `tf_mode` parameter of the `config` file.


### Alignment and MSA integration

To integrate the orthologue into the MSA, `42` chooses the most appropriate **template(s)** for alignment among the five closest relatives. As for taxonomic inference, it considers each of them in turn and stops once the coverage of the orthologue cannot be significantly improved. This allows `42` to select a slightly less related sequence as a template provided it aligns with a longer part of the orthologue. By how much exactly coverage has to be improved for a close sequence to be retained as a template can be fine-tuned with the `coverage_mul` parameter of the `config` file.

> **Experimental** — If the `patch_mode` parameter of the `config` file is set to `on`, close sequences belonging to the same `org` as the orthologue to be added cannot be selected as templates. This is to give new orthologues a chance to align better than these pre-existing sequences.

Then comes the alignment itself. With nucleotide `banks`, both `BLAST` and `exonerate` aligners are available, whereas only `BLAST` can be used with protein `banks`. The preferred aligner can be specified using the `aligner` parameter of the `config` file.

The `BLAST` aligner is the same as in the original `forty`. It extracts all the HSPs for the selected template(s) from the XML `BLAST` report and uses them as guides for integrating the orthologue's fragments into the MSA. Accessions in identifiers are modified so as to end with the rank of the template and the rank of the HSP (i.e., `XXXXXXXX.Ht.h`).

When the new `exonerate` aligner is preferred, only the longest selected template is used. The model successfully used by `exonerate` is appended to the accession: `E.lc` for `protein2genome` and `E.bf` for `protein2genome:bestfit --exhaustive`.

In most cases, the orthologue can be aligned as a single large fragment. If not, `42` emits different types of warnings depending on the exact issue. In worst cases (e.g., `exonerate` crashing), the orthologue cannot be integrated and has to be discarded. To avoid this, one can enable `BLAST` as a fall-back for exonerate failures by setting the `aligner` parameter to `exoblast`.

Aligned orthologues (possibly fragmented due to multiple HSPs) are integrated  into the MSA all at once but in the following arrangement: first by family, then by ascending position in the MSA, and finally by descending length. This is similar to what was done by the original `forty` but could be improved in the future, for example by physically regrouping paralogues that belong to the same family.


### Redundancy detection and handling

Independently of the aligner, `42` never integrates twice the same fragment for a given organism, even if obtained from multiple orthologues. Further, it filters out fragments included in sequences from the same organism that are either already present in the MSA or that are listed in the `.non` counterpart of the MSA. When an orthologue fragment includes a sequence already present in the MSA for the same organism, the latter can be either kept or removed, depending on the value of the parameter `ls_action` in the `config` file.

<!-- Je vais placer mes remarques sur les yaml ici.
    Pour tout ce qui est bank, dans orgs ou bien ref_org_mapper, je trouve que tu ne devrais pas mettre des noms de fichiers avec une extension .fa car de nouveaux utilisateurs pourraient ne pas comprendre qu'il faut créer une blast db.
    Pour blast_args, il faudrait dire que, si rien n'est précisé, tu prends les valeurs par défaut de BLAST.
    Pour ref_org_mul, tu ne reparles pas des ref_orgs qui sont enlevés s'il n'y a pas de hit via les query_orgs
    Plus vite une question qu'une remarque, a quel moment choisis-tu les fameux 10 premiers hits et 5 premiers hits pour bitscore_mul et coverage_mul car je n'ai rien observé dans le code.
    Pour le choix de template avec exonerate et blast, j'ai des gros doutes sur le faits que ca fonctionne de la meme manière ou bien que coverage_mul fonctionne sur les deux.
    Le fait est que les boucles ne se font pas sur la meme chose, ligne 357 pour exonerate on ne prend que un template hors de @template_seqs. Pour Blast, ligne 411 tu boucles sur orthologue->all_seqs mais ensuite tu prends quand meme dans @template_seqs ligne 414. Je ne vois pas d'explications à cela.
    Pour ==Engine to be used for aligning new seqs==, tu parles d'ALI alors que le user_guide parle de MSA
    Pour ==Taxonomic mode for identifying contaminations== taxon id est écrit en un seul mot ('taxonid')
    Juste avant - org: Asbestopluma hypogea, tu as écrit bacteria en micro-eukaryotes à la place de bacteria and micro-eukaryotes
    
 -->


# Annexes

## Command line interface

```none
\include{cli.txt}
```
\pagebreak

## Annotated YAML `config` file

```yaml
\include{../test/config-42-tax-exo.yaml}
```
\pagebreak

## Alternative simple YAML `config` file

```yaml
\include{../test/config-42-simple.yaml}
```
