package Bio::MUST::Tools::FortyTwo;
# ABSTRACT: Main class for forty-two tool
$Bio::MUST::Tools::FortyTwo::VERSION = '0.161640';
use Moose;
use namespace::autoclean;

use autodie;
use feature qw(say);

use Smart::Comments -ENV;

use Carp;

use Bio::FastParsers   0.151741;
use Bio::MUST::Core    0.151741;
use Bio::MUST::Drivers 0.151741;

use aliased 'Bio::MUST::Tools::FortyTwo::RunProcessor';


has 'config' => (
    traits   => ['Hash'],
    is       => 'ro',
    isa      => 'HashRef',
    required => 1,
    handles  => {
        args_for  => 'get',
    },
);


has 'infiles' => (
    traits   => ['Array'],
    is       => 'ro',
    isa      => 'ArrayRef[Str]',
    required => 1,
    handles  => {
        all_infiles => 'elements',
    },
);


sub run_proc {
    my $self = shift;
    my $args = shift // {};             # HashRef (should not be empty...)

    # do some pre-flight QC
    # and propagate 'defaults' args as default values for all org args
    my %def_args = %{ $self->args_for('defaults') };
    for my $org_args ( @{ $self->args_for('orgs') } ) {

        # check that org has no underscore between genus and species
        # to prevent 42 from seeing Genus_species as distinct from Genus species
        my $org = $org_args->{org};
        if ($org =~ m/^(\S+)_(.*)/xms) {
            carp "Warning: $org incorrectly written; use $1 $2 instead!";
        }

        $org_args = { %def_args, %{ $org_args } };
    }

    # combine YAML and CLI parameters (debug_mode, tax_reports)
    # Note: CLI take precedences over YAML (in case of duplicates)
    my %args = ( %{ $self->config }, %{$args} );
    # TODO: simplify this scheme?

    # add infiles
    $args{infiles} = $self->infiles;

    # TODO: improve sorting consistency when reporting org lists

    ### [42] Bio::FastParsers.....: $Bio::FastParsers::VERSION
    ### [42] Bio::MUST::Core......: $Bio::MUST::Core::VERSION
    ### [42] Bio::MUST::Drivers...: $Bio::MUST::Drivers::VERSION
    ### [42] Bio::MUST::Tools.....: $Bio::MUST::Tools::VERSION

    ### [42] config: %args

    return RunProcessor->new(%args);
}


__PACKAGE__->meta->make_immutable;
1;

__END__

=pod

=head1 NAME

Bio::MUST::Tools::FortyTwo - Main class for forty-two tool

=head1 VERSION

version 0.161640

=head1 SYNOPSIS

    # TODO

=head1 DESCRIPTION

    # TODO

=head1 AUTHOR

Denis BAURAIN <denis.baurain@ulg.ac.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by University of Liege / Unit of Eukaryotic Phylogenomics / Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
