package Bio::MUST::Tools::Leel;
# ABSTRACT: Main class for leel tool
$Bio::MUST::Tools::Leel::VERSION = '0.161640';
use Moose;
use namespace::autoclean;

use autodie;
use feature qw(say);

use Smart::Comments -ENV;

use Carp;

use Bio::FastParsers   0.153330;
use Bio::MUST::Core    0.153350;
use Bio::MUST::Drivers 0.153340;

use aliased 'Bio::MUST::Tools::Leel::RunProcessor';


has 'config' => (
    traits   => ['Hash'],
    is       => 'ro',
    isa      => 'HashRef',
    required => 1,
    handles  => {
        args_for  => 'get',
    },
);


has 'infiles' => (
    traits   => ['Array'],
    is       => 'ro',
    isa      => 'ArrayRef[Str]',
    required => 1,
    handles  => {
        all_infiles => 'elements',
    },
);

# TODO: check if all is necessary
# TODO: check how to avoid code duplication with 42

sub run_proc {
    my $self  = shift;
    my $debug = shift // 0;

    my %args = %{ $self->config };

    # do some pre-flight QC
    # and propagate 'defaults' args as default values for all org args
    my %def_args = %{ $self->args_for('defaults') };
    for my $org_args ( @{ $self->args_for('orgs') } ) {

        # check that org has no underscore between genus and species
        # to prevent 1331 from seeing Genus_species as distinct from Genus species
        my $org = $org_args->{org};
        if ($org =~ m/^(\S+)_(.*)/xms) {
            carp "Warning: $org incorrectly written; use $1 $2 instead!";
        }

        $org_args = { %def_args, %{ $org_args } };
    }

    # add infiles
    $args{infiles} = $self->infiles;

    # enable debugging mode
    $args{debug_mode} = $debug;

    ### [1331] config: %args

    return RunProcessor->new(%args);
}


__PACKAGE__->meta->make_immutable;
1;

__END__

=pod

=head1 NAME

Bio::MUST::Tools::Leel - Main class for leel tool

=head1 VERSION

version 0.161640

=head1 SYNOPSIS

    # TODO

=head1 DESCRIPTION

    # TODO

=head1 AUTHOR

Denis BAURAIN <denis.baurain@ulg.ac.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by University of Liege / Unit of Eukaryotic Phylogenomics / Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
