package Bio::MUST::Tools::Leel::RunProcessor;
# ABSTRACT: internal class for leel tool
$Bio::MUST::Tools::Leel::RunProcessor::VERSION = '0.161640';
use Moose;
use namespace::autoclean;

use autodie;
use feature qw(say);

use Smart::Comments;                    # logging always enabled here

use aliased 'Bio::MUST::Tools::Leel::AliProcessor';


has 'bank_dir' => (
    is       => 'ro',
    isa      => 'Str',
    required => 1,
);


has 'blast_args' => (
    traits   => ['Hash'],
    is       => 'ro',
    isa      => 'HashRef[HashRef]',
    default  => sub { {} },
    handles  => {
        blast_args_for => 'get',
    },
);


has 'id_match_mode' => (
    is       => 'ro',
    isa      => 'Str',
    default  => 'enforce',
);


has 'round_trip_mode' => (
    is       => 'ro',
    isa      => 'Str',
    default  => 'off',
);


has 'out_suffix' => (
    is       => 'ro',
    isa      => 'Maybe[Str]',
    default  => '-1331',
);


has 'orgs' => (
    traits   => ['Array'],
    is       => 'ro',
    isa      => 'ArrayRef[HashRef]',
    required => 1,
    handles  => {
        all_orgs => 'elements',
    },
);


has 'infiles' => (
    traits   => ['Array'],
    is       => 'ro',
    isa      => 'ArrayRef[Str]',
    required => 1,
    handles  => {
          all_infiles => 'elements',
    },
);


has 'debug_mode' => (
    is       => 'ro',
    isa      => 'Bool',
    default  => 0,
);


sub BUILD {
    my $self = shift;

    for my $infile ($self->all_infiles) {
        ### [RUN] Processing ALI: $infile
        AliProcessor->new(
            run_proc => $self,
            ali      => $infile,
        );
    }

    return;
}


__PACKAGE__->meta->make_immutable;
1;

__END__

=pod

=head1 NAME

Bio::MUST::Tools::Leel::RunProcessor - internal class for leel tool

=head1 VERSION

version 0.161640

=head1 AUTHOR

Denis BAURAIN <denis.baurain@ulg.ac.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by University of Liege / Unit of Eukaryotic Phylogenomics / Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
