package Bio::MUST::Tools::OmpaPa::Hmmer;
# ABSTRACT: internal class for tabular HMMER parser
$Bio::MUST::Tools::OmpaPa::Hmmer::VERSION = '0.161640';
use Moose;
use namespace::autoclean;

use autodie;
use feature qw(say);

use Smart::Comments;

use List::AllUtils qw(mesh);

extends 'Bio::FastParsers::Hmmer::Table';
with 'Bio::MUST::Tools::OmpaPa::Roles::Parsable';


sub collect_hits {
    my $self = shift;

    my @hits;

    # parse HMMER report
    while (my $hit = $self->next_hit) {

        # collect useful hit attributes
        push @hits, {
            'acc' => $hit->target_name,
            'dsc' => $hit->target_description // 'none',
            'exp' => $hit->evalue,
            'bit' => $hit->score,
        };
    }

    # fetch seq lengths
    my @ids  = map { $_->{acc}   } @hits;
    my $seqs = $self->_blastdb->blastdbcmd( \@ids );
    my @lens = map { $_->seq_len } $seqs->all_seqs;
    my %len_for = mesh @ids, @lens;
    ### assert: @ids == @lens

    # append seq lengths
    $_->{len} = $len_for{ $_->{acc} }
        for @hits;

    return \@hits;
}

__PACKAGE__->meta->make_immutable;
1;

__END__

=pod

=head1 NAME

Bio::MUST::Tools::OmpaPa::Hmmer - internal class for tabular HMMER parser

=head1 VERSION

version 0.161640

=head1 SYNOPSIS

    # TODO

=head1 DESCRIPTION

    # TODO

=head1 AUTHOR

Denis BAURAIN <denis.baurain@ulg.ac.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by University of Liege / Unit of Eukaryotic Phylogenomics / Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
