#!/usr/bin/env perl

use Test::Most;
use Test::Files;

use autodie;
use feature qw(say);

use Config::Any;
use Path::Class qw(file);

use Bio::MUST::Tools::FortyTwo;

my $class = 'Bio::MUST::Tools::FortyTwo';

check_forty('notax-blast');
check_forty('notax-blast-plus');
check_forty('tax-blast');
check_forty('tax-exo');
check_forty('simple');
check_forty('noalign');
check_forty('blastn', '-nt');

sub check_forty {
    my $variant = shift;
    my $suffix  = shift // q{};

    explain $variant;

    # read configuration file
    my $config_file = file('test', "config-42-$variant.yaml")->stringify;
    my $config = Config::Any->load_files( {
        files           => [ $config_file ],
        flatten_to_hash => 1,
        use_ext         => 1,
    } );

    # build ft object
    my $ft = $class->new(
        config  => $config->{$config_file},
        infiles => [
            file('test', "rpl2$suffix.ali"    )->stringify,
            file('test', "rpl2-del$suffix.ali")->stringify,
            file('test', "rpl3$suffix.ali"    )->stringify,
        ],
    );

    # use ft as factory for run_proc object
    my $run_proc = $ft->run_proc;
    # TODO: test tax reports

    compare_ok(
        file('test', "rpl2$suffix-my-42-$variant.ali"),
        file('test', "rpl2$suffix-42-$variant.ali"),
            "wrote expected Ali for: rpl2 ($suffix-$variant)"
    );

    compare_ok(
        file('test', "rpl2-del$suffix-my-42-$variant.ali"),
        file('test', "rpl2-del$suffix-42-$variant.ali"),
            "wrote expected Ali for: rpl2-del ($suffix-$variant)"
    );

    compare_ok(
        file('test', "rpl3$suffix-my-42-$variant.ali"),
        file('test', "rpl3$suffix-42-$variant.ali"),
            "wrote expected Ali for: rpl3 ($suffix-$variant)"
    );
}

done_testing;
