#!/usr/bin/env perl

use Test::Most;
use Test::Files;

use autodie;
use feature qw(say);

use Config::Any;
use Path::Class qw(file);

use Bio::MUST::Tools::Leel;

my $class = 'Bio::MUST::Tools::Leel';

check_leel('simple');

sub check_leel {
    my $variant = shift;

    explain $variant;

    # read configuration file
    my $config_file = file('test', "config-1331-$variant.yaml")->stringify;
    my $config = Config::Any->load_files( {
        files           => [ $config_file ],
        flatten_to_hash => 1,
        use_ext         => 1,
    } );

    # build leel object
    my $leel = $class->new(
        config  => $config->{$config_file},
        infiles => [
            file('test', 'rpl2-new.ali'    )->stringify,
            file('test', 'rpl2-del-new.ali')->stringify,
            file('test', 'rpl3-new.ali'    )->stringify,
        ],
    );

    # use leel as factory for run_proc object
    my $run_proc = $leel->run_proc;

    compare_ok(
        file('test', "rpl2-new-my-1331-$variant.ali"),
        file('test', "rpl2-new-1331-$variant.ali"),
            "wrote expected Ali for: rpl2 ($variant)"
    );

    compare_ok(
        file('test', "rpl2-del-new-my-1331-$variant.ali"),
        file('test', "rpl2-del-new-1331-$variant.ali"),
            "wrote expected Ali for: rpl2-del ($variant)"
    );

    compare_ok(
        file('test', "rpl3-new-my-1331-$variant.ali"),
        file('test', "rpl3-new-1331-$variant.ali"),
            "wrote expected Ali for: rpl3 ($variant)"
    );
}

done_testing;
