#!/usr/bin/env perl

use Test::Most;
use Test::Files;
use Test::Output;

use autodie;
use feature qw(say);

use Path::Class qw(file);

use Bio::MUST::Tools::OmpaPa;

{
    my $class = 'Bio::MUST::Tools::OmpaPa::Hmmer';

    my $oum = $class->new(
        file         => file('test', 'hmmsearch.out'),
        extract_seqs => 1,
        database     => file('test', 'database'),
    );
    isa_ok $oum, $class;
    cmp_ok $oum->count_hits, '==', 241, 'processed expected number of hits';

    $oum->restore_bounds;
    cmp_ok $oum->selection, '==', 5, 'got expected number of selected hits';

    stdout_is { $oum->list_selection } <<'EOT', 'got expected selection list';
======================================================================
accession                                   description length evalue 
----------------------------------------------------------------------
Sulfurimonas_autotrophica_563040@ADN08900   none        675    5.1e-46
Tolumonas_auensis_595494@ACQ92628           none        533    1.5e-43
Hippea_maritima_760142@AEA33167             none        698    1.6e-43
Deferribacter_desulfuricans_639282@BAI80200 none        709    2.5e-40
Aminobacterium_colombiense_572547@ADE56604  none        704    5.5e-39
======================================================================
EOT

    $oum->save_selection;
    compare_ok(
        file('test', 'hmmsearch.idl'),
        file('test', 'hmmsearch.exp-idl'),  # TODO: use 'my_' naming scheme
            'wrote expected IDL file for hmmsearch.out'
    );
    compare_ok(
        file('test', 'hmmsearch.fasta'),
        file('test', 'hmmsearch.exp-fasta'),
            'wrote expected FASTA file for hmmsearch.out'
    );
}

done_testing;
